@@ -2,6 +2,7 @@
 
 from __future__ import absolute_import, print_function
 
+import pipes
 import time
 
 import libcloud.compute.base as cloud_base
@@ -37,7 +38,7 @@ class ComputeNodeDriver(BaseComputeNodeDriver):
             auth_kwargs, list_kwargs, create_kwargs,
             driver_class)
 
-    def arvados_create_kwargs(self, arvados_node):
+    def arvados_create_kwargs(self, size, arvados_node):
         cluster_id, _, node_id = arvados_node['uuid'].split('-')
         name = 'compute-{}-{}'.format(node_id, cluster_id)
         tags = {
@@ -45,9 +46,20 @@ class ComputeNodeDriver(BaseComputeNodeDriver):
             'arv-ping-url': self._make_ping_url(arvados_node)
         }
         tags.update(self.tags)
+
+        customdata = """#!/bin/sh
+mkdir -p    /var/tmp/arv-node-data/meta-data
+echo %s > /var/tmp/arv-node-data/arv-ping-url
+echo %s > /var/tmp/arv-node-data/meta-data/instance-id
+echo %s > /var/tmp/arv-node-data/meta-data/instance-type
+""" % (pipes.quote(tags['arv-ping-url']),
+       pipes.quote(name),
+       pipes.quote(size.id))
+
         return {
             'name': name,
             'ex_tags': tags,
+            'ex_customdata': customdata
         }
 
     def sync_node(self, cloud_node, arvados_node):
@@ -61,20 +73,6 @@ class ComputeNodeDriver(BaseComputeNodeDriver):
     def _init_image(self, urn):
         return "image", self.get_image(urn)
 
-    def post_create_node(self, cloud_node):
-        self.real.ex_run_command(cloud_node,
-                                 """bash -c '
-                                 mkdir -p /var/tmp/arv-node-data/meta-data
-                                 echo "%s" > /var/tmp/arv-node-data/arv-ping-url
-                                 echo "%s" > /var/tmp/arv-node-data/meta-data/instance-id
-                                 echo "%s" > /var/tmp/arv-node-data/meta-data/instance-type
-                                 echo "%s" > /var/tmp/arv-node-data/meta-data/local-ipv4
-                                 '""" % (cloud_node.extra["tags"]["arv-ping-url"],
-                                         cloud_node.id,
-                                         cloud_node.extra["properties"]["hardwareProfile"]["vmSize"],
-                                         cloud_node.private_ips[0]),
-                                 timestamp=int(time.time()))
-
     def list_nodes(self):
         # Azure only supports filtering node lists by resource group.
         # Do our own filtering based on tag.
